#
# Copyright (c) 2025, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    List,
    Type,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.time_series_lineage import TimeSeriesLineage
from ..models.time_series_lineage_entity_type import TimeSeriesLineageEntityType
from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
    from ..models.attributes_holder_identifier import AttributesHolderIdentifier


T = TypeVar("T", bound="TimeSeries")


@_attrs_define
class TimeSeries:
    """
    Attributes:
        attribute (str):
        holder (AttributesHolderIdentifier):
        include_preview (Union[Unset, bool]):
        lineage (Union[Unset, TimeSeriesLineage]):
        lineage_entity_type (Union[Unset, TimeSeriesLineageEntityType]):
        normalize_to_first_value (Union[Unset, bool]):
    """

    attribute: str
    holder: "AttributesHolderIdentifier"
    include_preview: Union[Unset, bool] = UNSET
    lineage: Union[Unset, TimeSeriesLineage] = UNSET
    lineage_entity_type: Union[Unset, TimeSeriesLineageEntityType] = UNSET
    normalize_to_first_value: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        attribute = self.attribute

        holder = self.holder.to_dict()

        include_preview = self.include_preview

        lineage: Union[Unset, str] = UNSET
        if not isinstance(self.lineage, Unset):
            lineage = self.lineage.value

        lineage_entity_type: Union[Unset, str] = UNSET
        if not isinstance(self.lineage_entity_type, Unset):
            lineage_entity_type = self.lineage_entity_type.value

        normalize_to_first_value = self.normalize_to_first_value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "attribute": attribute,
                "holder": holder,
            }
        )
        if include_preview is not UNSET:
            field_dict["includePreview"] = include_preview
        if lineage is not UNSET:
            field_dict["lineage"] = lineage
        if lineage_entity_type is not UNSET:
            field_dict["lineageEntityType"] = lineage_entity_type
        if normalize_to_first_value is not UNSET:
            field_dict["normalizeToFirstValue"] = normalize_to_first_value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.attributes_holder_identifier import AttributesHolderIdentifier

        d = src_dict.copy()
        attribute = d.pop("attribute")

        holder = AttributesHolderIdentifier.from_dict(d.pop("holder"))

        include_preview = d.pop("includePreview", UNSET)

        _lineage = d.pop("lineage", UNSET)
        lineage: Union[Unset, TimeSeriesLineage]
        if isinstance(_lineage, Unset):
            lineage = UNSET
        else:
            lineage = TimeSeriesLineage(_lineage)

        _lineage_entity_type = d.pop("lineageEntityType", UNSET)
        lineage_entity_type: Union[Unset, TimeSeriesLineageEntityType]
        if isinstance(_lineage_entity_type, Unset):
            lineage_entity_type = UNSET
        else:
            lineage_entity_type = TimeSeriesLineageEntityType(_lineage_entity_type)

        normalize_to_first_value = d.pop("normalizeToFirstValue", UNSET)

        time_series = cls(
            attribute=attribute,
            holder=holder,
            include_preview=include_preview,
            lineage=lineage,
            lineage_entity_type=lineage_entity_type,
            normalize_to_first_value=normalize_to_first_value,
        )

        time_series.additional_properties = d
        return time_series

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#
# Copyright (c) 2025, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    List,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
    from ..models.query_leaderboard_params_field_dto import QueryLeaderboardParamsFieldDTO
    from ..models.query_leaderboard_params_opened_group_with_pagination_params_dto import (
        QueryLeaderboardParamsOpenedGroupWithPaginationParamsDTO,
    )


T = TypeVar("T", bound="QueryLeaderboardParamsGroupingParamsDTO")


@_attrs_define
class QueryLeaderboardParamsGroupingParamsDTO:
    """
    Attributes:
        group_by (List['QueryLeaderboardParamsFieldDTO']):
        opened_groups (Union[Unset, List[str]]):
        opened_groups_with_pagination (Union[Unset, List['QueryLeaderboardParamsOpenedGroupWithPaginationParamsDTO']]):
    """

    group_by: List["QueryLeaderboardParamsFieldDTO"]
    opened_groups: Union[Unset, List[str]] = UNSET
    opened_groups_with_pagination: Union[Unset, List["QueryLeaderboardParamsOpenedGroupWithPaginationParamsDTO"]] = (
        UNSET
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        group_by = []
        for group_by_item_data in self.group_by:
            group_by_item = group_by_item_data.to_dict()
            group_by.append(group_by_item)

        opened_groups: Union[Unset, List[str]] = UNSET
        if not isinstance(self.opened_groups, Unset):
            opened_groups = self.opened_groups

        opened_groups_with_pagination: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.opened_groups_with_pagination, Unset):
            opened_groups_with_pagination = []
            for opened_groups_with_pagination_item_data in self.opened_groups_with_pagination:
                opened_groups_with_pagination_item = opened_groups_with_pagination_item_data.to_dict()
                opened_groups_with_pagination.append(opened_groups_with_pagination_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "groupBy": group_by,
            }
        )
        if opened_groups is not UNSET:
            field_dict["openedGroups"] = opened_groups
        if opened_groups_with_pagination is not UNSET:
            field_dict["openedGroupsWithPagination"] = opened_groups_with_pagination

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.query_leaderboard_params_field_dto import QueryLeaderboardParamsFieldDTO
        from ..models.query_leaderboard_params_opened_group_with_pagination_params_dto import (
            QueryLeaderboardParamsOpenedGroupWithPaginationParamsDTO,
        )

        d = src_dict.copy()
        group_by = []
        _group_by = d.pop("groupBy")
        for group_by_item_data in _group_by:
            group_by_item = QueryLeaderboardParamsFieldDTO.from_dict(group_by_item_data)

            group_by.append(group_by_item)

        opened_groups = cast(List[str], d.pop("openedGroups", UNSET))

        opened_groups_with_pagination: Union[Unset, List[QueryLeaderboardParamsOpenedGroupWithPaginationParamsDTO]] = (
            UNSET
        )
        _opened_groups_with_pagination = d.pop("openedGroupsWithPagination", UNSET)
        if not isinstance(_opened_groups_with_pagination, Unset):
            opened_groups_with_pagination = []
            for opened_groups_with_pagination_item_data in _opened_groups_with_pagination:
                opened_groups_with_pagination_item = QueryLeaderboardParamsOpenedGroupWithPaginationParamsDTO.from_dict(
                    opened_groups_with_pagination_item_data
                )

                opened_groups_with_pagination.append(opened_groups_with_pagination_item)

        query_leaderboard_params_grouping_params_dto = cls(
            group_by=group_by,
            opened_groups=opened_groups,
            opened_groups_with_pagination=opened_groups_with_pagination,
        )

        query_leaderboard_params_grouping_params_dto.additional_properties = d
        return query_leaderboard_params_grouping_params_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#
# Copyright (c) 2025, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum


class ApiErrorTypeDTO(str, Enum):
    ACTIVE_PROJECTS_QUOTA_COULD_NOT_BE_CHANGED = "ACTIVE_PROJECTS_QUOTA_COULD_NOT_BE_CHANGED"
    ADMIN_CANNOT_LEAVE_PROJECT = "ADMIN_CANNOT_LEAVE_PROJECT"
    CANNOT_ADD_MANAGER_TO_INDIVIDUAL_PROJECT = "CANNOT_ADD_MANAGER_TO_INDIVIDUAL_PROJECT"
    CANNOT_ADD_MEMBERS_TO_PRIVATE_INDIVIDUAL_PROJECT = "CANNOT_ADD_MEMBERS_TO_PRIVATE_INDIVIDUAL_PROJECT"
    CANNOT_ADD_OUTSIDERS_TO_TEAM_PROJECT = "CANNOT_ADD_OUTSIDERS_TO_TEAM_PROJECT"
    CANNOT_LEAVE_TEAM_PROJECT = "CANNOT_LEAVE_TEAM_PROJECT"
    CANNOT_MANAGE_ORGANIZATION_OWNER_IN_PROJECT = "CANNOT_MANAGE_ORGANIZATION_OWNER_IN_PROJECT"
    INSUFFICIENT_ACTIVE_PROJECTS_QUOTA = "INSUFFICIENT_ACTIVE_PROJECTS_QUOTA"
    LIMIT_OF_ACTIVE_PROJECTS_QUOTA_REACHED = "LIMIT_OF_ACTIVE_PROJECTS_QUOTA_REACHED"
    LIMIT_OF_ACTIVE_PROJECTS_REACHED = "LIMIT_OF_ACTIVE_PROJECTS_REACHED"
    LIMIT_OF_MEMBERS_IN_ORGANIZATION_REACHED = "LIMIT_OF_MEMBERS_IN_ORGANIZATION_REACHED"
    LIMIT_OF_PROJECTS_REACHED = "LIMIT_OF_PROJECTS_REACHED"
    LIMIT_OF_PROJECT_MEMBERS_REACHED = "LIMIT_OF_PROJECT_MEMBERS_REACHED"
    LIMIT_OF_SERVICE_ACCOUNTS_IN_ORGANIZATION_REACHED = "LIMIT_OF_SERVICE_ACCOUNTS_IN_ORGANIZATION_REACHED"
    PAYMENTS_NOT_SUPPORTED = "PAYMENTS_NOT_SUPPORTED"
    PLAN_NOT_AVAILABLE = "PLAN_NOT_AVAILABLE"
    PROJECT_KEY_COLLISION = "PROJECT_KEY_COLLISION"
    PROJECT_KEY_INVALID = "PROJECT_KEY_INVALID"
    PROJECT_NAME_COLLISION = "PROJECT_NAME_COLLISION"
    PROJECT_NAME_INVALID = "PROJECT_NAME_INVALID"
    PROJECT_VISIBILITY_NOT_SUPPORTED_FOR_IAAC = "PROJECT_VISIBILITY_NOT_SUPPORTED_FOR_IAAC"
    SERVICE_ACCOUNT_ALREADY_EXISTS_IN_ORGANIZATION = "SERVICE_ACCOUNT_ALREADY_EXISTS_IN_ORGANIZATION"
    SERVICE_ACCOUNT_NOT_IN_PROJECT = "SERVICE_ACCOUNT_NOT_IN_PROJECT"
    USER_NOT_IN_PROJECT = "USER_NOT_IN_PROJECT"
    USER_STILL_OWNS_SOME_ORGANIZATIONS = "USER_STILL_OWNS_SOME_ORGANIZATIONS"
    VISIBILITY_RESTRICTED = "VISIBILITY_RESTRICTED"
    WORKSPACE_READONLY = "WORKSPACE_READONLY"

    def __str__(self) -> str:
        return str(self.value)



#
# Copyright (c) 2025, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    List,
    Type,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.attribute_definition_dto import AttributeDefinitionDTO
    from ..models.next_page_dto import NextPageDTO


T = TypeVar("T", bound="QueryAttributeDefinitionsResultDTO")


@_attrs_define
class QueryAttributeDefinitionsResultDTO:
    """
    Attributes:
        entries (List['AttributeDefinitionDTO']):
        next_page (NextPageDTO):
    """

    entries: List["AttributeDefinitionDTO"]
    next_page: "NextPageDTO"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        entries = []
        for entries_item_data in self.entries:
            entries_item = entries_item_data.to_dict()
            entries.append(entries_item)

        next_page = self.next_page.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "entries": entries,
                "nextPage": next_page,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.attribute_definition_dto import AttributeDefinitionDTO
        from ..models.next_page_dto import NextPageDTO

        d = src_dict.copy()
        entries = []
        _entries = d.pop("entries")
        for entries_item_data in _entries:
            entries_item = AttributeDefinitionDTO.from_dict(entries_item_data)

            entries.append(entries_item)

        next_page = NextPageDTO.from_dict(d.pop("nextPage"))

        query_attribute_definitions_result_dto = cls(
            entries=entries,
            next_page=next_page,
        )

        query_attribute_definitions_result_dto.additional_properties = d
        return query_attribute_definitions_result_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#
# Copyright (c) 2025, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import (
    Any,
    Dict,
    List,
    Type,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.api_error_type_dto import ApiErrorTypeDTO
from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="Error")


@_attrs_define
class Error:
    """
    Attributes:
        code (int):
        message (str):
        type (Union[Unset, ApiErrorTypeDTO]):
        error_type (Union[Unset, ApiErrorTypeDTO]):
    """

    code: int
    message: str
    type: Union[Unset, ApiErrorTypeDTO] = UNSET
    error_type: Union[Unset, ApiErrorTypeDTO] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        code = self.code

        message = self.message

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        error_type: Union[Unset, str] = UNSET
        if not isinstance(self.error_type, Unset):
            error_type = self.error_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "code": code,
                "message": message,
            }
        )
        if type is not UNSET:
            field_dict["type"] = type
        if error_type is not UNSET:
            field_dict["errorType"] = error_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = d.pop("code")

        message = d.pop("message")

        _type = d.pop("type", UNSET)
        type: Union[Unset, ApiErrorTypeDTO]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = ApiErrorTypeDTO(_type)

        _error_type = d.pop("errorType", UNSET)
        error_type: Union[Unset, ApiErrorTypeDTO]
        if isinstance(_error_type, Unset):
            error_type = UNSET
        else:
            error_type = ApiErrorTypeDTO(_error_type)

        error = cls(
            code=code,
            message=message,
            type=type,
            error_type=error_type,
        )

        error.additional_properties = d
        return error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#
# Copyright (c) 2025, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import (
    Any,
    Dict,
    List,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.permission import Permission
from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="FileToSign")


@_attrs_define
class FileToSign:
    """
    Attributes:
        path (str):
        permission (Permission):
        project_identifier (str):
        size (Union[None, Unset, int]):
    """

    path: str
    permission: Permission
    project_identifier: str
    size: Union[None, Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        path = self.path

        permission = self.permission.value

        project_identifier = self.project_identifier

        size: Union[None, Unset, int]
        if isinstance(self.size, Unset):
            size = UNSET
        else:
            size = self.size

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "path": path,
                "permission": permission,
                "project_identifier": project_identifier,
            }
        )
        if size is not UNSET:
            field_dict["size"] = size

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        path = d.pop("path")

        permission = Permission(d.pop("permission"))

        project_identifier = d.pop("project_identifier")

        def _parse_size(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        size = _parse_size(d.pop("size", UNSET))

        file_to_sign = cls(
            path=path,
            permission=permission,
            project_identifier=project_identifier,
            size=size,
        )

        file_to_sign.additional_properties = d
        return file_to_sign

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing
if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions
DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _ProtoScale:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ProtoScaleEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ProtoScale.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    linear: _ProtoScale.ValueType
    log: _ProtoScale.ValueType

class ProtoScale(_ProtoScale, metaclass=_ProtoScaleEnumTypeWrapper):
    ...
linear: ProtoScale.ValueType
log: ProtoScale.ValueType
global___ProtoScale = ProtoScale

class _ProtoLineage:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ProtoLineageEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ProtoLineage.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    FULL: _ProtoLineage.ValueType
    ONLY_OWNED: _ProtoLineage.ValueType

class ProtoLineage(_ProtoLineage, metaclass=_ProtoLineageEnumTypeWrapper):
    """Enum for lineage types"""
FULL: ProtoLineage.ValueType
ONLY_OWNED: ProtoLineage.ValueType
global___ProtoLineage = ProtoLineage

class _LineageEntityType:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _LineageEntityTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_LineageEntityType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    EXPERIMENT: _LineageEntityType.ValueType
    RUN: _LineageEntityType.ValueType

class LineageEntityType(_LineageEntityType, metaclass=_LineageEntityTypeEnumTypeWrapper):
    ...
EXPERIMENT: LineageEntityType.ValueType
RUN: LineageEntityType.ValueType
global___LineageEntityType = LineageEntityType

@typing.final
class ProtoGetTimeseriesBucketsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    EXPRESSIONS_FIELD_NUMBER: builtins.int
    VIEW_FIELD_NUMBER: builtins.int

    @property
    def expressions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ProtoCustomExpression]:
        ...

    @property
    def view(self) -> global___ProtoView:
        ...

    def __init__(self, *, expressions: collections.abc.Iterable[global___ProtoCustomExpression] | None=..., view: global___ProtoView | None=...) -> None:
        ...

    def HasField(self, field_name: typing.Literal['view', b'view']) -> builtins.bool:
        ...

    def ClearField(self, field_name: typing.Literal['expressions', b'expressions', 'view', b'view']) -> None:
        ...
global___ProtoGetTimeseriesBucketsRequest = ProtoGetTimeseriesBucketsRequest

@typing.final
class ProtoCustomExpression(google.protobuf.message.Message):
    """Custom metric configuration"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    REQUESTID_FIELD_NUMBER: builtins.int
    RUNID_FIELD_NUMBER: builtins.int
    CUSTOMYFORMULA_FIELD_NUMBER: builtins.int
    INCLUDEPREVIEW_FIELD_NUMBER: builtins.int
    LINEAGE_FIELD_NUMBER: builtins.int
    ENTITYTYPE_FIELD_NUMBER: builtins.int
    requestId: builtins.str
    'needs to be unique in the whole ProtoGetTimeseriesBucketsRequest'
    runId: builtins.str
    customYFormula: builtins.str
    includePreview: builtins.bool
    lineage: global___ProtoLineage.ValueType
    entityType: global___LineageEntityType.ValueType

    def __init__(self, *, requestId: builtins.str=..., runId: builtins.str=..., customYFormula: builtins.str=..., includePreview: builtins.bool | None=..., lineage: global___ProtoLineage.ValueType=..., entityType: global___LineageEntityType.ValueType=...) -> None:
        ...

    def HasField(self, field_name: typing.Literal['_includePreview', b'_includePreview', 'includePreview', b'includePreview']) -> builtins.bool:
        ...

    def ClearField(self, field_name: typing.Literal['_includePreview', b'_includePreview', 'customYFormula', b'customYFormula', 'entityType', b'entityType', 'includePreview', b'includePreview', 'lineage', b'lineage', 'requestId', b'requestId', 'runId', b'runId']) -> None:
        ...

    def WhichOneof(self, oneof_group: typing.Literal['_includePreview', b'_includePreview']) -> typing.Literal['includePreview'] | None:
        ...
global___ProtoCustomExpression = ProtoCustomExpression

@typing.final
class ProtoView(google.protobuf.message.Message):
    """View configuration"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    FROM_FIELD_NUMBER: builtins.int
    TO_FIELD_NUMBER: builtins.int
    POINTFILTERS_FIELD_NUMBER: builtins.int
    MAXBUCKETS_FIELD_NUMBER: builtins.int
    XSCALE_FIELD_NUMBER: builtins.int
    YSCALE_FIELD_NUMBER: builtins.int
    XAXIS_FIELD_NUMBER: builtins.int
    to: builtins.float
    maxBuckets: builtins.int
    xScale: global___ProtoScale.ValueType
    yScale: global___ProtoScale.ValueType

    @property
    def pointFilters(self) -> global___ProtoPointFilters:
        ...

    @property
    def xAxis(self) -> global___ProtoXAxis:
        ...

    def __init__(self, *, to: builtins.float | None=..., pointFilters: global___ProtoPointFilters | None=..., maxBuckets: builtins.int=..., xScale: global___ProtoScale.ValueType=..., yScale: global___ProtoScale.ValueType=..., xAxis: global___ProtoXAxis | None=...) -> None:
        ...

    def HasField(self, field_name: typing.Literal['_from', b'_from', '_pointFilters', b'_pointFilters', '_to', b'_to', 'from', b'from', 'pointFilters', b'pointFilters', 'to', b'to', 'xAxis', b'xAxis']) -> builtins.bool:
        ...

    def ClearField(self, field_name: typing.Literal['_from', b'_from', '_pointFilters', b'_pointFilters', '_to', b'_to', 'from', b'from', 'maxBuckets', b'maxBuckets', 'pointFilters', b'pointFilters', 'to', b'to', 'xAxis', b'xAxis', 'xScale', b'xScale', 'yScale', b'yScale']) -> None:
        ...

    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal['_from', b'_from']) -> typing.Literal['from'] | None:
        ...

    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal['_pointFilters', b'_pointFilters']) -> typing.Literal['pointFilters'] | None:
        ...

    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal['_to', b'_to']) -> typing.Literal['to'] | None:
        ...
global___ProtoView = ProtoView

@typing.final
class ProtoPointFilters(google.protobuf.message.Message):
    """Point filters"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    STEPRANGE_FIELD_NUMBER: builtins.int

    @property
    def stepRange(self) -> global___ProtoOpenRange:
        ...

    def __init__(self, *, stepRange: global___ProtoOpenRange | None=...) -> None:
        ...

    def HasField(self, field_name: typing.Literal['_stepRange', b'_stepRange', 'stepRange', b'stepRange']) -> builtins.bool:
        ...

    def ClearField(self, field_name: typing.Literal['_stepRange', b'_stepRange', 'stepRange', b'stepRange']) -> None:
        ...

    def WhichOneof(self, oneof_group: typing.Literal['_stepRange', b'_stepRange']) -> typing.Literal['stepRange'] | None:
        ...
global___ProtoPointFilters = ProtoPointFilters

@typing.final
class ProtoOpenRange(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    FROM_FIELD_NUMBER: builtins.int
    TO_FIELD_NUMBER: builtins.int
    to: builtins.float

    def __init__(self, *, to: builtins.float | None=...) -> None:
        ...

    def HasField(self, field_name: typing.Literal['_from', b'_from', '_to', b'_to', 'from', b'from', 'to', b'to']) -> builtins.bool:
        ...

    def ClearField(self, field_name: typing.Literal['_from', b'_from', '_to', b'_to', 'from', b'from', 'to', b'to']) -> None:
        ...

    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal['_from', b'_from']) -> typing.Literal['from'] | None:
        ...

    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal['_to', b'_to']) -> typing.Literal['to'] | None:
        ...
global___ProtoOpenRange = ProtoOpenRange

@typing.final
class ProtoXAxis(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    STEPS_FIELD_NUMBER: builtins.int
    EPOCHMILLIS_FIELD_NUMBER: builtins.int
    RELATIVETIME_FIELD_NUMBER: builtins.int
    CUSTOM_FIELD_NUMBER: builtins.int

    @property
    def steps(self) -> global___XSteps:
        ...

    @property
    def epochMillis(self) -> global___XEpochMillis:
        ...

    @property
    def relativeTime(self) -> global___XRelativeTime:
        ...

    @property
    def custom(self) -> global___XCustom:
        ...

    def __init__(self, *, steps: global___XSteps | None=..., epochMillis: global___XEpochMillis | None=..., relativeTime: global___XRelativeTime | None=..., custom: global___XCustom | None=...) -> None:
        ...

    def HasField(self, field_name: typing.Literal['custom', b'custom', 'epochMillis', b'epochMillis', 'relativeTime', b'relativeTime', 'steps', b'steps', 'value', b'value']) -> builtins.bool:
        ...

    def ClearField(self, field_name: typing.Literal['custom', b'custom', 'epochMillis', b'epochMillis', 'relativeTime', b'relativeTime', 'steps', b'steps', 'value', b'value']) -> None:
        ...

    def WhichOneof(self, oneof_group: typing.Literal['value', b'value']) -> typing.Literal['steps', 'epochMillis', 'relativeTime', 'custom'] | None:
        ...
global___ProtoXAxis = ProtoXAxis

@typing.final
class XSteps(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None:
        ...
global___XSteps = XSteps

@typing.final
class XEpochMillis(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None:
        ...
global___XEpochMillis = XEpochMillis

@typing.final
class XRelativeTime(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None:
        ...
global___XRelativeTime = XRelativeTime

@typing.final
class XCustom(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    EXPRESSION_FIELD_NUMBER: builtins.int
    expression: builtins.str

    def __init__(self, *, expression: builtins.str=...) -> None:
        ...

    def ClearField(self, field_name: typing.Literal['expression', b'expression']) -> None:
        ...
global___XCustom = XCustom


#
# Copyright (c) 2025, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import (
    Any,
    Dict,
    List,
    Type,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ProjectDTO")


@_attrs_define
class ProjectDTO:
    """
    Attributes:
        name (str):
        organization_name (str):
        version (int):
        id (str):
        project_key (str):
        organization_id (str):
    """

    name: str
    organization_name: str
    version: int
    id: str
    project_key: str
    organization_id: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        organization_name = self.organization_name

        version = self.version

        id = self.id

        project_key = self.project_key

        organization_id = self.organization_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "organizationName": organization_name,
                "version": version,
                "id": id,
                "projectKey": project_key,
                "organizationId": organization_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        organization_name = d.pop("organizationName")

        version = d.pop("version")

        id = d.pop("id")

        project_key = d.pop("projectKey")

        organization_id = d.pop("organizationId")

        project_dto = cls(
            name=name,
            organization_name=organization_name,
            version=version,
            id=id,
            project_key=project_key,
            organization_id=organization_id,
        )

        project_dto.additional_properties = d
        return project_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#
# Copyright (c) 2025, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Contains shared errors types that can be raised from API functions"""

import httpx

__all__ = [
    "UnexpectedStatus",
    "InvalidApiTokenException",
    "UnableToExchangeApiKeyError",
    "ApiKeyRejectedError",
    "UnableToDeserializeApiKeyError",
    "UnableToParseResponse",
    "UnableToRefreshTokenError",
]


class UnexpectedStatus(Exception):
    """Raised by api functions when the response status an undocumented status and Client.raise_on_unexpected_status is True"""

    def __init__(self, status_code: int, content: bytes):
        self.status_code = status_code
        self.content = content

        super().__init__(
            f"Unexpected status code: {status_code}\n\nResponse content:\n{content.decode(errors='ignore')}"
        )


class InvalidApiTokenException(Exception):
    """Raised when the api token is invalid"""

    def __init__(self, reason: str = "") -> None:
        super().__init__(f"Invalid API token. Reason: {reason}")


class UnableToExchangeApiKeyError(Exception):
    """Raised when the API key exchange fails for any reason other than the API key being
    explicitly rejected by the server"""

    def __init__(self, reason: str = "Unknown") -> None:
        super().__init__(f"Unable to exchange API key. Reason: {reason}")


class ApiKeyRejectedError(Exception):
    """Raised when the backend rejects an API key because of it being unknown or expired"""

    def __init__(self) -> None:
        super().__init__("Your API key was rejected by the Neptune backend because it is either unknown or expired.")


class UnableToDeserializeApiKeyError(Exception):
    """Raised when the API key cannot be deserialized"""

    def __init__(self) -> None:
        super().__init__("Unable to deserialize API key")


class UnableToRefreshTokenError(Exception):
    """Raised when the token refresh fails"""

    def __init__(self, reason: str = "Unknown") -> None:
        super().__init__(f"Unable to refresh token. Reason: {reason}")


class UnableToParseResponse(Exception):
    """Raise when there is an exception during parsing a response"""

    def __init__(self, exception: BaseException, response: httpx.Response) -> None:
        self.exception = exception
        self.response = response
        super().__init__(
            f"Unable to parse server response: {exception}. "
            f"Response: HTTP {response.status_code}: {response.content!r}"
        )



#
# Copyright (c) 2025, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import (
    Any,
    Dict,
    List,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="AttributeNameFilterDTO")


@_attrs_define
class AttributeNameFilterDTO:
    """
    Attributes:
        must_match_any (Union[Unset, List['AttributeNameFilterDTO']]): A list of filters representing disjunctions of
            conjunctive simple filters. The implementation is currently limited to 1 level of nesting. E.g. `(a AND b AND
            NOT c) OR (d AND e) OR (f AND g)` is supported, but `(a OR (b OR c))` is not supported. The latter should be
            expressed as `(a OR b OR c)`. `(a AND (b OR c))` is also not supported. Mutually exclusive with
            `mustMatchRegexes` and `mustNotMatchRegexes`.
        must_match_regexes (Union[Unset, List[str]]): An attribute must match all of the regexes from the list to be
            returned. Mutually exclusive with `mustMatchAny`
        must_not_match_regexes (Union[Unset, List[str]]): An attribute must match none of the regexes from the list to
            be returned. Mutually exclusive with `mustMatchAny`
    """

    must_match_any: Union[Unset, List["AttributeNameFilterDTO"]] = UNSET
    must_match_regexes: Union[Unset, List[str]] = UNSET
    must_not_match_regexes: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        must_match_any: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.must_match_any, Unset):
            must_match_any = []
            for must_match_any_item_data in self.must_match_any:
                must_match_any_item = must_match_any_item_data.to_dict()
                must_match_any.append(must_match_any_item)

        must_match_regexes: Union[Unset, List[str]] = UNSET
        if not isinstance(self.must_match_regexes, Unset):
            must_match_regexes = self.must_match_regexes

        must_not_match_regexes: Union[Unset, List[str]] = UNSET
        if not isinstance(self.must_not_match_regexes, Unset):
            must_not_match_regexes = self.must_not_match_regexes

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if must_match_any is not UNSET:
            field_dict["mustMatchAny"] = must_match_any
        if must_match_regexes is not UNSET:
            field_dict["mustMatchRegexes"] = must_match_regexes
        if must_not_match_regexes is not UNSET:
            field_dict["mustNotMatchRegexes"] = must_not_match_regexes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        must_match_any: Union[Unset, List[AttributeNameFilterDTO]] = UNSET
        _must_match_any = d.pop("mustMatchAny", UNSET)
        if not isinstance(_must_match_any, Unset):
            must_match_any = []
            for must_match_any_item_data in _must_match_any:
                must_match_any_item = AttributeNameFilterDTO.from_dict(must_match_any_item_data)

                must_match_any.append(must_match_any_item)

        must_match_regexes = cast(List[str], d.pop("mustMatchRegexes", UNSET))

        must_not_match_regexes = cast(List[str], d.pop("mustNotMatchRegexes", UNSET))

        attribute_name_filter_dto = cls(
            must_match_any=must_match_any,
            must_match_regexes=must_match_regexes,
            must_not_match_regexes=must_not_match_regexes,
        )

        attribute_name_filter_dto.additional_properties = d
        return attribute_name_filter_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
